# Generated by Django 5.0.4 on 2024-05-25 19:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mozilla_django_oidc_db", "0002_migrate_to_claim_field"),
    ]

    operations = [
        migrations.AddField(
            model_name="openidconnectconfig",
            name="oidc_keycloak_idp_hint",
            field=models.CharField(
                blank=True,
                help_text="Specific for Keycloak: parameter that indicates which identity provider should be used (therefore skipping the Keycloak login screen).",
                max_length=1000,
                verbose_name="Keycloak Identity Provider hint",
            ),
        ),
        migrations.AddField(
            model_name="openidconnectconfig",
            name="oidc_op_logout_endpoint",
            field=models.URLField(
                blank=True,
                help_text="URL of your OpenID Connect provider logout endpoint",
                max_length=1000,
                verbose_name="Logout endpoint",
            ),
        ),
    ]
