# Generated by Django 4.2.20 on 2025-04-11 14:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mozilla_django_oidc_db", "0007_migrate_oidc_data"),
    ]

    operations = [
        migrations.DeleteModel(
            name="OpenIDConnectConfig",
        ),
        migrations.CreateModel(
            name="OpenIDConnectConfig",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "managed": False,
            },
        ),
    ]
